@@ -3,6 +3,8 @@
   
   (c) Simon Marlow 1990-1993
   (c) Albert Graef 1994
+
+  modified 2005, 2006 by Bernhard R. Link (see Changelog)
 -----------------------------------------------------------------------------*/
 
 #ifndef AM_H
@@ -18,12 +20,10 @@
 typedef struct {
 	char *name;
 	char *directory;
-	char *fname;
 	char *icon;
 	char *push_action;
 	char *drop_action;
 	Pixmap icon_bm;
-	Boolean loaded;
 	Widget form, toggle, label;
 	Boolean selected;
 } AppRec, *AppList;
@@ -31,37 +31,41 @@ typedef struct {
 typedef struct {
 	Widget shell, form, viewport, button_box, icon_box;
 	AppList apps;
-	int n_apps;
-	int n_selections;
+	Cardinal n_apps;
+	Cardinal n_selections;
+	char *app_file;
+	Boolean readonly;
 } AppWindowRec;
  
 extern AppWindowRec aw;
-extern Widget app_popup_widget, *app_popup_items, app_popup_widget1;
+extern Widget app_popup_widget, *app_popup_items,
+              app_popup_widget1, *app_popup_items1;
 
 extern int n_appst;
 extern char **appst;
 
-Pixmap defaultIcon(char *name, char *directory, char *fname);
-int parseApp(FILE *fp, char **name, char **directory, char **fname,
-	     char **icon, char **push_action, char **drop_action);
-void createApplicationWindow();
-void createApplicationDisplay();
-void updateApplicationDisplay();
-void setApplicationWindowName();
-void readApplicationData(String path);
-int writeApplicationData(String path);
-void installApplication(char *name, char *directory, char *fname, char *icon,
-			char *push_action, char *drop_action);
-void replaceApplication(AppRec *app, char *name, char *directory, char *fname,
-			char *icon, char *push_action, char *drop_action);
+int parseApp(FILE *fp, Boolean loadicons);
+void createApplicationWindow(void);
+void createApplicationDisplay(void);
+void updateApplicationDisplay(void);
+void setApplicationWindowName(void);
+void readApplicationData(void);
+int writeApplicationData(void);
+void installApplication(const char *name, const char *directory,
+                        const char *icon,
+                        const char *push_action, const char *drop_action);
+void replaceApplication(AppRec *app, const char *name, const char *directory,
+                        const char *icon,
+			const char *push_action, const char *drop_action);
 void freeApplicationResources(AppRec *app);
-void pushApplicationsFile();
-void popApplicationsFile();
-void clearApplicationsStack();
+void popApplicationsFile(void);
+void clearApplicationsStack(void);
 
+void AmReadUpdate(void);
+Boolean AmPushLoadFile(const char *filename, Boolean fromFW);
 /*--FmAwCb-------------------------------------------------------------------*/
 
-FmCallbackProc 
+CallbackProc 
   appInstallAppCb, appInstallGroupCb, appEditCb, appCutCb, appCopyCb,
   appPasteCb, appRemoveCb, appSelectAllCb, appDeselectCb, appLoadCb,
   appMainCb, appBackCb, appOpenCb, appCloseCb, aboutCb;
@@ -70,24 +74,19 @@ FmCallbackProc
 
 int  findAppWidget(Widget w);
 
-FmActionProc appPopup, appMaybeHighlight, runApp, appSelect, appToggle;
+FmActionProc appPopup, appMaybeHighlight, runApp, appSelect, appToggle, appTrackCursor;
 
 void appBeginDrag(Widget w, XEvent *event, String *params, 
 		  Cardinal *num_params);
 
-void appEndMove(int i);
-void appEndMoveInBox(void);
-void appEndCopy(int i);
-void appEndCopyInBox(void);
-
 void appEndDrag(int i);
 void appEndDragInBox(void);
 
 /*--FmAwPopup----------------------------------------------------------------*/
 
-void installNewAppPopup();
-void installExistingAppPopup();
-void installGroupPopup();
-void createInstallPopups();
+void installNewAppPopup(void);
+void installExistingAppPopup(void);
+void installGroupPopup(void);
+void createInstallPopups(void);
 
 #endif
