@@ -241,7 +241,10 @@ void FileListReduceFilenames(FileListWidget flw, int max_width) {
  for (i=0; i<FL.n_files; i++) {
  	file=FL.files[i];
 
-	assert(file->reducedname == NULL);
+	/* when switching quickly from iconfilelist to textfilelist
+	 * and back, those might still be set */
+	if( file->reducedname != NULL )
+		free(file->reducedname);
 
 	namelen = strlen(file->name);
 	name = XtMalloc(namelen + 1);
