@@ -5,7 +5,7 @@
  *  ------------------------------------------------------------------
  
  *  Copyright (C) 1997  by Till Straumann   <strauman@sun6hft.ee.tu-berlin.de>
- *  modified 2006 by Bernhard R. Link (see Changelog)
+ *  modified 2006,2007 by Bernhard R. Link (see Changelog)
 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ typedef struct {
     FileList		files;
     int			n_files;
     Pixel		foreground,highlight_pixel;
-    Dimension		border_width; 
+    Dimension		border_width;
     XtPointer		user_data;
     XtCallbackList	notify_cbl;
     int			drag_timeout;
@@ -67,4 +67,7 @@ typedef struct _FileListRec {
     FileListPart	fileList;
 } FileListRec;
 
+/* reduce all names to have length at most max_width */
+void FileListReduceFilenames(FileListWidget flw, int max_width);
+
 #endif /* _FileListP_h */
