@@ -558,6 +558,10 @@ Boolean cmpCanonical(const char *dir1, const char *dir2);
 int create(const char *path, mode_t mode), rcopy(const char *oldpath, const char *newpath),
   rmove(const char *oldpath, const char *newpath), rdel(const char *path);
 
+/*--FmViewportScroll---------------------------------------------------------*/
+
+void initViewportScroll(XtAppContext context);
+
 /*---------------------------------------------------------------------------*/
 
 /* Horrible kludge to avoid warnings, as XtFree is designed to take a (char *)*/
