@@ -1,14 +1,34 @@
-/*-----------------------------------------------------------------------------
+/*
+ *  This file is part of X-File Manager XFM
+ *  ----------------------------------------------------------------------
   FmAw.c
   
   (c) Simon Marlow 1990-1993
   (c) Albert Graef 1994
 
   modified 7-1997 by strauman@sun6hft.ee.tu-berlin.de to add
-  different enhancements (see README-NEW).
+  different enhancements (see README-1.4).
+
+  modified 2003,2005,2006 by Bernhard R. Link (see Changelog)
 
   Functions & data for creating & maintaining  the application window
------------------------------------------------------------------------------*/
+ *  ----------------------------------------------------------------------
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <xfmconfig.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -18,11 +38,13 @@
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
 #include <X11/StringDefs.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Box.h>
-#include <X11/Xaw/Viewport.h>
-#include <X11/Xaw/Toggle.h>
-#include <X11/Xaw/Label.h>
+#include <X11/Xaw3d/Form.h>
+#include <X11/Xaw3d/Box.h>
+#include <X11/Xaw3d/Viewport.h>
+#include <X11/Xaw3d/Toggle.h>
+#include <X11/Xaw3d/Label.h>
+
+#include "global.h"
 
 #include "Am.h"
 
@@ -38,7 +60,7 @@
 -----------------------------------------------------------------------------*/
 AppWindowRec aw;
 Widget *aw_button_items,
-app_popup_widget, *app_popup_items, app_popup_widget1;
+app_popup_widget, *app_popup_items, app_popup_widget1, *app_popup_items1;
 int n_appst = 0;
 char **appst = NULL;
 
@@ -47,7 +69,7 @@ char **appst = NULL;
   STATIC DATA 
 -----------------------------------------------------------------------------*/
 
-static MenuItemRec app_popup_menu[] = {
+static const MenuItemRec app_popup_menu[] = {
   { "install app", "Install...", appInstallAppCb },
   { "install group", "Install group...", appInstallGroupCb },
   { "line1", NULL, NULL },
@@ -65,7 +87,7 @@ static MenuItemRec app_popup_menu[] = {
   { "quit", "Quit", appCloseCb },
 };
 
-static MenuItemRec app_popup_menu1[] = {
+static const MenuItemRec app_popup_menu1[] = {
   { "edit", "Edit...", appEditCb },
   { "line1", NULL, NULL },
   { "cut", "Cut", appCutCb },
@@ -148,8 +170,9 @@ static char app_translations[] = "\
 static char iconbox_translations[] = "\
     <Btn2Up>            : dummy()\n\
     <Btn3Up>            : dummy()\n\
-    <Btn3Down>          : appPopup()\n";
-
+    Any <Btn3Down>      : appPopup()\n\
+    <Enter>             : appTrackCursor()\n\
+    <Leave>             : appTrackCursor()\n";
 #endif
 
 /*-----------------------------------------------------------------------------
@@ -157,8 +180,9 @@ static char iconbox_translations[] = "\
 -----------------------------------------------------------------------------*/
 static void CatchEntryLeave2(Widget,XtPointer,XEvent*,Boolean *ctd);
 
-static void dummy(Widget w, XEvent *event, String *params, 
-		       Cardinal *num_params) {}
+static void dummy(UNUSED(Widget w), UNUSED(XEvent *event),
+                  UNUSED(String *params), UNUSED(Cardinal *num_params)) {
+}
 
 static XtActionsRec app_actions[] = {
   { "appMaybeHighlight", appMaybeHighlight },
@@ -167,6 +191,7 @@ static XtActionsRec app_actions[] = {
   { "appToggle", appToggle },
   { "appPopup", appPopup },
   { "appBeginDrag", appBeginDrag },
+  { "appTrackCursor", appTrackCursor },
   { "dummy", dummy }
 };
 
@@ -174,10 +199,10 @@ static XtActionsRec app_actions[] = {
   Button Data
 -----------------------------------------------------------------------------*/
 
-static ButtonRec aw_buttons[] = {
-  { "back", "Back", (FmCallbackProc *) appBackCb },
-  { "main", "Main", (FmCallbackProc *) appMainCb },
-  { "reload", "Reload", (FmCallbackProc *) appLoadCb },
+static const ButtonRec aw_buttons[] = {
+  { "back", "Back", appBackCb },
+  { "main", "Main", appMainCb },
+  { "reload", "Reload", appLoadCb },
   { "open", "File window", appOpenCb },
 };
 
@@ -186,9 +211,10 @@ static ButtonRec aw_buttons[] = {
   PRIVATE FUNCTIONS
 -----------------------------------------------------------------------------*/
 
-static int longestName()
+static int longestName(void)
 {
-  int i, l, longest = 0;
+  int l, longest = 0;
+  Cardinal i;
 
   for (i=0; i<aw.n_apps; i++)
     if ((l = XTextWidth(resources.icon_font, aw.apps[i].name, 
@@ -197,12 +223,9 @@ static int longestName()
   return longest;
 }
 
+/*---------------------------------------------------------------------------*/
 
-/*-----------------------------------------------------------------------------
-  PUBLIC FUNCTIONS
------------------------------------------------------------------------------*/
-
-int parseApp(FILE *fp, char **name, char **directory, char **fname,
+static int parseAppI(FILE *fp, char **name, char **directory, char **fname,
 	     char **icon, char **push_action, char **drop_action)
 {
   static char s[MAXAPPSTRINGLEN];
@@ -212,6 +235,7 @@ int parseApp(FILE *fp, char **name, char **directory, char **fname,
   if (feof(fp)||!fgets(s, MAXAPPSTRINGLEN, fp))
     return 0;
   l = strlen(s);
+  /* TODO: check for overlong lines */
   if (s[l-1] == '\n')
     s[--l] = '\0';
   if (!l || *s == '#')
@@ -233,59 +257,123 @@ int parseApp(FILE *fp, char **name, char **directory, char **fname,
 
 /*---------------------------------------------------------------------------*/
 
-/* determine the default icon of an application */
+/* determine the icon of an application */
 
-Pixmap defaultIcon(char *name, char *directory, char *fname)
-{
-  if (!*fname)
-    return bm[EXEC_BM];
-  else {
-    struct stat stats;
-    Boolean sym_link;
-    int l = *directory?strlen(directory):strlen(user.home);
-    char *path = (char *)alloca(2+strlen(fname)+l);
-    
-    strcpy(path, *directory?directory:user.home);
+static void readApplicationBitmap(AppRec *app, Boolean interactive) {
 
-    if (l) {
-      if (path[l-1] != '/')
-	path[l++] = '/';
-      strcpy(path+l, fname);
-    } else
-      strcpy(path, fname);
+	app->icon_bm = None;
 
-    if (lstat(path, &stats))
-      return bm[FILE_BM];
-    else if (S_ISLNK(stats.st_mode)) {
-      sym_link = True;
-      stat(path, &stats);
-    } else
-      sym_link = False;
+	if( app->icon[0] != '\0' ) {
+		app->icon_bm = loadIcon(app->icon,NULL,NULL);
+		if( app->icon_bm == None ) {
+			if( interactive )
+				error("Can't read icon for", app->name);
+			else
+				fprintf(stderr, 
+"%s: can't read icon for application %s\n", progname, app->name);
+		}
+	}
+	if( app->icon_bm == None ) {
+		/* choose a default icon, no longer on file type but
+		 * on what happens with that */
+		if( strcmp(app->push_action,"LOAD") == 0 ) {
+			app->icon_bm = bm_appmgr;
+		} else if( strcmp(app->push_action,"OPEN") == 0 ) {
+			app->icon_bm = bm[DIR_BM];
+		} else
+			app->icon_bm = bm[FILE_BM];
+	}
+}
 
-    if (S_ISLNK(stats.st_mode))
-      return bm[BLACKHOLE_BM];
-    else if (S_ISDIR(stats.st_mode))
-      if (sym_link)
-	return bm[DIRLNK_BM];
-      else if (!strcmp(name, ".."))
-	return bm[UPDIR_BM];
-      else
-	return bm[DIR_BM];
-    else if (stats.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
-      if (sym_link)
-	return bm[EXECLNK_BM];
-      else
-	return bm[EXEC_BM];
-    else if (sym_link)
-      return bm[SYMLNK_BM];
-    else
-      return bm[FILE_BM];
-  }
+/*---------------------------------------------------------------------------*/
+
+static void generateDeviceApp(DevRec *dev) {
+	char *name;
+
+	name = strrchr(dev->name,'/');
+	if (name == NULL)
+		name = dev->name;
+	else 
+		name++;
+
+	aw.apps = (AppList) XTREALLOC(aw.apps, (aw.n_apps+1)*sizeof(AppRec) );
+	aw.apps[aw.n_apps].name = XtNewString(name);
+	aw.apps[aw.n_apps].directory = XtNewString(dev->name);
+
+	aw.apps[aw.n_apps].icon = XtNewString(dev->type);
+	aw.apps[aw.n_apps].push_action = XtNewString("OPEN");
+	aw.apps[aw.n_apps].drop_action = XtNewString("");
+	aw.n_apps++;
+}
+
+/*-----------------------------------------------------------------------------
+  PUBLIC FUNCTIONS
+-----------------------------------------------------------------------------*/
+
+int parseApp(FILE *fp, Boolean loadicons) {
+	char *name, *directory, *fname, *icon, *push_action, *drop_action;
+	char s[MAXAPPSTRINGLEN];
+	int p;
+
+	p = parseAppI(fp, &name, &directory, &fname, &icon, 
+			&push_action, &drop_action);
+	if( p <= 0 )
+		return p;
+	aw.apps = (AppList) XTREALLOC(aw.apps, (aw.n_apps+1)*sizeof(AppRec) );
+	aw.apps[aw.n_apps].name = XtNewString(strparse(s, name, "\\:"));
+	aw.apps[aw.n_apps].directory = XtNewString(strparse(s, directory, 
+				"\\:"));
+	aw.apps[aw.n_apps].icon = XtNewString(strparse(s, icon, "\\:"));
+	aw.apps[aw.n_apps].push_action = XtNewString(strparse(s, push_action,
+				"\\:"));
+	aw.apps[aw.n_apps].drop_action = XtNewString(strparse(s, drop_action,
+				"\\:"));
+	/* look if it is old-format and try to fix it */
+	fname = strparse(s, fname, "\\:");
+	push_action = aw.apps[aw.n_apps].push_action;
+	if( fname[0] != '\0' ) {
+		if( strcmp(push_action,"LOAD") == 0 ) {
+			char *appfile = dirConcat(
+					aw.apps[aw.n_apps].directory,
+					fname);
+			XtFree(aw.apps[aw.n_apps].directory);
+			aw.apps[aw.n_apps].directory = appfile;
+		} else if( strcmp(push_action,"OPEN") == 0 ) {
+			if( strcmp(fname,".") == 0 ) {
+				/* everything already OK */
+			} else if( aw.apps[aw.n_apps].directory[0] == '\0' ) {
+				char *dir = dirConcat("~",fname);
+				XtFree(aw.apps[aw.n_apps].directory);
+				aw.apps[aw.n_apps].directory = dir;
+			} else {
+				char *dir = dirConcat(
+						aw.apps[aw.n_apps].directory,
+						fname);
+				XtFree(aw.apps[aw.n_apps].directory);
+				aw.apps[aw.n_apps].directory = dir;
+			}
+		} else if( push_action[0] != '\0' ) {
+			aw.apps[aw.n_apps].push_action = spaceConcat(
+					push_action, fname);
+			XtFree(push_action);
+		}
+		if( drop_action[0] != '\0' ) {
+			char *action = spaceConcat(
+					aw.apps[aw.n_apps].drop_action,
+					fname);
+			XtFree(aw.apps[aw.n_apps].drop_action);
+			aw.apps[aw.n_apps].drop_action = action;
+		}
+	}
+	if( loadicons )
+		readApplicationBitmap(&aw.apps[aw.n_apps],False);
+	aw.n_apps++;
+	return p;
 }
 
 /*-------------------------------------------------------------------------*/
 
-void createApplicationWindow()
+void createApplicationWindow(void)
 {
 #ifndef ENHANCE_TRANSLATIONS
   XtTranslations t;
@@ -313,7 +401,7 @@ void createApplicationWindow()
 				       XtNumber(app_popup_menu),
 				       4, aw.shell,
 				       NULL, &app_popup_widget);
-  createFloatingMenu("app popup 1", app_popup_menu1,
+  app_popup_items1 = createFloatingMenu("app popup 1", app_popup_menu1,
 		     XtNumber(app_popup_menu1), 4, aw.shell,
 		     NULL, &app_popup_widget1);
 
@@ -322,7 +410,7 @@ void createApplicationWindow()
 
   /* create the form */
 #ifndef ENHANCE_TRANSLATIONS
-  aw.form = XtCreateManagedWidget("form", formWidgetClass, aw.shell,
+  aw.form = XtCreateManagedWidget("awform", formWidgetClass, aw.shell,
 #else
   aw.form = XtCreateManagedWidget("awform", formWidgetClass, aw.shell,
 #endif
@@ -345,9 +433,9 @@ void createApplicationWindow()
 
 /*---------------------------------------------------------------------------*/
 
-void createApplicationDisplay()
+void createApplicationDisplay(void)
 {
-  int i;
+  Cardinal i;
   Dimension width;
 
   for (i=0; i<aw.n_apps; i++)
@@ -419,7 +507,7 @@ XtInsertEventHandler(aw.apps[i].toggle,
 
 /*---------------------------------------------------------------------------*/
 
-void updateApplicationDisplay()
+void updateApplicationDisplay(void)
 {
   XtDestroyWidget(aw.icon_box);
   createApplicationDisplay();
@@ -428,85 +516,100 @@ void updateApplicationDisplay()
 
 /*---------------------------------------------------------------------------*/
 
-void setApplicationWindowName()
+void setApplicationWindowName(void)
 {
-  char app_file_name[MAXPATHLEN], *p;
-  if ((p = strrchr(resources.app_file, '/')))
-    strcpy(app_file_name, p+1);
+  char *p;
+  if ((p = strrchr(aw.app_file, '/')))
+    p++;
   else
-    strcpy(app_file_name, resources.app_file);
-  XSetIconName(XtDisplay(aw.shell), XtWindow(aw.shell), app_file_name);
-  XStoreName(XtDisplay(aw.shell), XtWindow(aw.shell), app_file_name);
+    p = aw.app_file;
+  XSetIconName(XtDisplay(aw.shell), XtWindow(aw.shell), p);
+  XStoreName(XtDisplay(aw.shell), XtWindow(aw.shell), p);
 }
 
 /*---------------------------------------------------------------------------*/
 
-void readApplicationBitmaps()
-{
-  int i;
-
-  for (i=0; i<aw.n_apps; i++) {
-    aw.apps[i].loaded = False;
-    if (!aw.apps[i].icon[0])
-      aw.apps[i].icon_bm = defaultIcon(aw.apps[i].name, aw.apps[i].directory,
-				       aw.apps[i].fname);
-    else if ((aw.apps[i].icon_bm = readIcon(aw.apps[i].icon,0,0)) == None) {
-      fprintf(stderr, "%s: can't read icon for application %s\n",
-	      progname, aw.apps[i].name);
-      aw.apps[i].icon_bm = defaultIcon(aw.apps[i].name, aw.apps[i].directory,
-				       aw.apps[i].fname);
-    } else
-      aw.apps[i].loaded = True;
-  }
-}
-
-/*---------------------------------------------------------------------------*/
-
-void readApplicationData(String path)
+void readApplicationData(void)
 {
   FILE *fp;
-  char *name, *directory, *fname, *icon, *push_action, *drop_action;
-  char s[MAXAPPSTRINGLEN];
-  int i, p;
+  int p;
+  Cardinal i;
   
   aw.n_apps = 0;
   aw.apps = NULL;
-  
-  if (!(fp = fopen(path, "r"))) return;
 
-  for (i=0; (p = parseApp(fp, &name, &directory, &fname, &icon, &push_action,
-			  &drop_action)) > 0; i++) {
-    aw.apps = (AppList) XTREALLOC(aw.apps, (i+1)*sizeof(AppRec) );
-    aw.apps[i].name = XtNewString(strparse(s, name, "\\:"));
-    aw.apps[i].directory = XtNewString(strparse(s, directory, "\\:"));
-    aw.apps[i].fname = XtNewString(strparse(s, fname, "\\:"));
-    aw.apps[i].icon = XtNewString(strparse(s, icon, "\\:"));
-    aw.apps[i].push_action = XtNewString(strparse(s, push_action, "\\:"));
-    aw.apps[i].drop_action = XtNewString(strparse(s, drop_action, "\\:"));
+  if (strcmp(aw.app_file,"XFM-DEVICES") == 0) {
+
+	  for (i=0; i<n_devices; i++) {
+		  generateDeviceApp(devs + i);
+	  }
+	  for (i=0; i<aw.n_apps; i++) {
+		  readApplicationBitmap(&aw.apps[i],False);
+	  }
+	  aw.readonly = True;
+	  return;
   }
+  
+  if (!(fp = fopen(aw.app_file, "r"))) return;
+
+  do { 
+	  p = parseApp(fp,False);
+  } while( p > 0 );
 
   if (p == -1)
     error("Error in applications file", "");
 
-  aw.n_apps = i;
-  
   if (fclose(fp))
     sysError("Error reading applications file:");
 
-  readApplicationBitmaps();
+  aw.readonly = False;
+
+  for (i=0; i<aw.n_apps; i++) {
+    readApplicationBitmap(&aw.apps[i],False);
+  }
 }
 
 /*---------------------------------------------------------------------------*/
 
-int writeApplicationData(String path)
+int writeApplicationData(void)
 {
   FILE *fp;
-  int i;
-  char name[2*MAXAPPSTRINGLEN], directory[2*MAXAPPSTRINGLEN],
-    fname[2*MAXAPPSTRINGLEN], icon[2*MAXAPPSTRINGLEN],
-    push_action[2*MAXAPPSTRINGLEN], drop_action[2*MAXAPPSTRINGLEN];
+  Cardinal i;
+
+  if (aw.readonly)
+	  return 0;
   
-  if (! (fp = fopen(path, "w") )) {
+  if (! (fp = fopen(aw.app_file, "w") )) {
+#ifdef SYSTEMWIDE_DEFAULTS
+    size_t app_dir_len = strlen(resources.app_dir);
+    if ( app_dir_len > 0 &&
+          strcasecmp(resources.app_dir,"none") != 0) {
+      size_t len_basename;
+      const char *p;char *n;
+
+      if ((p = strrchr(aw.app_file, '/')))
+	p++;
+      else
+        p = aw.app_file;
+      len_basename = strlen(p);
+      n = XtMalloc(app_dir_len+len_basename+2);
+      memcpy(n, resources.app_dir, app_dir_len);
+      n[app_dir_len] = '/';
+      memcpy(n+app_dir_len+1, p, len_basename+1);
+
+      if (!exists(resources.app_dir)){
+        mkdir(resources.app_dir,0777);
+      }
+      if ((fp = fopen(n, "w") )) {
+        XTFREE(aw.app_file);
+	aw.app_file = n;
+      } else {
+	XTFREE(n);
+      }
+    }
+#endif
+  }
+  if (! fp ) {
     sysError("Error writing applications file:");
     return -1;
   }
@@ -514,14 +617,17 @@ int writeApplicationData(String path)
   fprintf(fp, "#XFM\n");
 
   for (i=0; i < aw.n_apps; i++) {
-    expand(name, aw.apps[i].name, "\\:");
-    expand(directory, aw.apps[i].directory, "\\:");
-    expand(fname, aw.apps[i].fname, "\\:");
-    expand(icon, aw.apps[i].icon, "\\:");
-    expand(push_action, aw.apps[i].push_action, "\\:");
-    expand(drop_action, aw.apps[i].drop_action, "\\:");
-    fprintf(fp, "%s:%s:%s:%s:%s:%s\n", name, directory, fname, icon,
-	    push_action, drop_action);
+    fprintexpand(fp, aw.apps[i].name, "\\:");
+    fputc(':',fp);
+    fprintexpand(fp, aw.apps[i].directory, "\\:");
+    fputc(':',fp);
+    fputc(':',fp);
+    fprintexpand(fp, aw.apps[i].icon, "\\:");
+    fputc(':',fp);
+    fprintexpand(fp, aw.apps[i].push_action, "\\:");
+    fputc(':',fp);
+    fprintexpand(fp, aw.apps[i].drop_action, "\\:");
+    fputc('\n',fp);
   }
   
   if (fclose(fp)) {
@@ -536,11 +642,8 @@ int writeApplicationData(String path)
 
 void freeApplicationResources(AppRec *app)
 {
-  if (app->loaded)
-    XFreePixmap(XtDisplay(aw.shell), app->icon_bm);
   XTFREE(app->name);
   XTFREE(app->directory);
-  XTFREE(app->fname);
   XTFREE(app->icon);
   XTFREE(app->push_action);
   XTFREE(app->drop_action);
@@ -548,87 +651,83 @@ void freeApplicationResources(AppRec *app)
 
 /*---------------------------------------------------------------------------*/
 
-void installApplication(char *name, char *directory, char *fname, char *icon,
-			char *push_action, char *drop_action)
+void installApplication(const char *name, const char *directory,
+	const char *icon, const char *push_action,
+	const char *drop_action)
 {
-  int i;
-  Pixmap icon_bm;
-  Boolean loaded = False;
 
-  if (!*icon)
-    icon_bm = defaultIcon(name, directory, fname);
-  else if ((icon_bm = readIcon(icon,0,0)) == None) {
-    error("Can't read icon for", name);
-    icon_bm = defaultIcon(name, directory, fname);
-  } else
-    loaded = True;
+  aw.apps = (AppList) XTREALLOC(aw.apps, (aw.n_apps+1) * sizeof(AppRec));
 
-  i = aw.n_apps++;
-  aw.apps = (AppList) XTREALLOC(aw.apps, aw.n_apps * sizeof(AppRec));
-
-  aw.apps[i].name = XtNewString(name);
-  aw.apps[i].directory = XtNewString(directory);
-  aw.apps[i].fname = XtNewString(fname);
-  aw.apps[i].icon = XtNewString(icon);
-  aw.apps[i].push_action = XtNewString(push_action);
-  aw.apps[i].drop_action = XtNewString(drop_action);
-  aw.apps[i].icon_bm = icon_bm;
-  aw.apps[i].loaded = loaded;
-  aw.apps[i].form = aw.apps[i].toggle = aw.apps[i].label = NULL;
+  aw.apps[aw.n_apps].name = XtNewString(name);
+  aw.apps[aw.n_apps].directory = XtNewString(directory);
+  aw.apps[aw.n_apps].icon = XtNewString(icon);
+  aw.apps[aw.n_apps].push_action = XtNewString(push_action);
+  aw.apps[aw.n_apps].drop_action = XtNewString(drop_action);
+  aw.apps[aw.n_apps].form = aw.apps[aw.n_apps].toggle = 
+	  		    aw.apps[aw.n_apps].label = NULL;
+  readApplicationBitmap(&aw.apps[aw.n_apps],True);
+  /* only increment here so that readApplicationBitmap can show error 
+   * messages without this part already visible */
+  aw.n_apps++;
 }
 
 /*---------------------------------------------------------------------------*/
 
-void replaceApplication(AppRec *app, char *name, char *directory, char *fname,
-			char *icon, char *push_action, char *drop_action)
+void replaceApplication(AppRec *app, const char *name, const char *directory,
+		const char *icon, const char *push_action,
+		const char *drop_action)
 {
-  Pixmap icon_bm;
-  Boolean loaded = False;
-
-  if (!*icon)
-    icon_bm = defaultIcon(name, directory, fname);
-  else if ((icon_bm = readIcon(icon,0,0)) == None) {
-    error("Can't read icon for", name);
-    icon_bm = defaultIcon(name, directory, fname);
-  } else
-    loaded = True;
+  /* This is not the nicest way, but it works, as 
+   * freeApplicationResources no longer frees or unsets app->icon_bm,
+   * readApplicationBitmap can be called interactive even afterwards, 
+   * as the old icon will still be visible - brl */
 
   freeApplicationResources(app);
 
   app->name = XtNewString(name);
   app->directory = XtNewString(directory);
-  app->fname = XtNewString(fname);
   app->icon = XtNewString(icon);
   app->push_action = XtNewString(push_action);
   app->drop_action = XtNewString(drop_action);
-  app->icon_bm = icon_bm;
-  app->loaded = loaded;
   app->form = app->toggle = app->label = NULL;
+  readApplicationBitmap(app,True);
 }
 
 /*---------------------------------------------------------------------------*/
 
-void pushApplicationsFile()
+static void pushApplicationsFile(const char *filename)
 {
-  int i = n_appst++;
+	int i = n_appst++;
 
-  appst = (char **) XTREALLOC(appst, n_appst * sizeof(char *));
-  appst[i] = XtNewString(resources.app_file);
+	appst = (char **) XTREALLOC(appst, n_appst * sizeof(char *));
+	appst[i] = aw.app_file;
+
+	if (filename[0] != '/' && strcmp(filename,"XFM-DEVICES") != 0) {
+		char *fn = dirConcat(resources.app_dir, filename);
+		if (exists(fn))
+			aw.app_file = fn;
+		else {
+			XtFree(fn);
+			aw.app_file = 
+				dirConcat(resources.system_app_dir, filename);
+		}
+	} else
+		aw.app_file = XtNewString(filename);
 }
 
 /*---------------------------------------------------------------------------*/
 
-void popApplicationsFile()
+void popApplicationsFile(void)
 {
   if (n_appst <= 0) return;
-  strcpy(resources.app_file, appst[--n_appst]);
-  XTFREE(appst[n_appst]);
+  XTFREE(aw.app_file);
+  aw.app_file = appst[--n_appst];
   appst = (char **) XTREALLOC(appst, n_appst * sizeof(char *));
 }
 
 /*---------------------------------------------------------------------------*/
 
-void clearApplicationsStack()
+void clearApplicationsStack(void)
 {
   int i;
   for (i = 0; i < n_appst; i++)
@@ -644,7 +743,7 @@ void clearApplicationsStack()
 */
 
 
-void CatchEntryLeave2(Widget w, XtPointer cld, XEvent* ev, Boolean *ctd)
+static void CatchEntryLeave2(UNUSED(Widget w), UNUSED(XtPointer cld), XEvent* ev, Boolean *ctd)
 {
 /*
     RBW - eat extraneous LeaveNotify/EnterNotify events caused by some window
@@ -660,4 +759,23 @@ return;
 }
 
 
+void AmReadUpdate(void) {
+	Cardinal i;
 
+	for(i=0; i<aw.n_apps; i++)
+		freeApplicationResources(&aw.apps[i]);
+	XTFREE(aw.apps);
+
+	readApplicationData();
+	updateApplicationDisplay();
+}
+
+Boolean AmPushLoadFile(const char *filename,Boolean fromFW) {
+
+	if( !resources.appmgr && !fromFW )
+		return False;
+
+	pushApplicationsFile(filename);
+	AmReadUpdate();
+	return True;
+}
