@@ -44,16 +44,19 @@
   STATIC DATA
 -----------------------------------------------------------------------------*/
 
-static Widget shell, label1, label2, label3;
-static enum { DontKnow, Ok, Cancel, Abort } confirm_flag;
-Boolean aborted;
+struct ConfirmationDialog {
+	Widget shell, label1, label2, label3;
+	enum { DontKnow, Ok, Cancel, Abort } confirm_flag;
+	Boolean aborted;
+};
 
 /*-----------------------------------------------------------------------------
   Widget Argument Lists
 -----------------------------------------------------------------------------*/
 
 static Arg shell_args[] = {
-  { XtNtitle, (XtArgVal) "Confirm" }
+  { XtNtitle, (XtArgVal) "Confirm" },
+  { XtNtransientFor, (XtArgVal) NULL }
 };
 
 static Arg *form_args = NULL;
@@ -84,29 +87,32 @@ static Arg button_box_args[] = {
   PRIVATE FUNCTIONS
 -----------------------------------------------------------------------------*/
 
-static void confirmOkCb(UNUSED(Widget w), UNUSED(void *client_data),
-		        UNUSED(void *call_data))
-{
-  XtPopdown(shell);
-  confirm_flag = Ok;
+static void confirmOkCb(UNUSED(Widget w), void *client_data,
+		        UNUSED(void *call_data)) {
+	struct ConfirmationDialog *d = client_data;
+
+	XtPopdown(d->shell);
+	d->confirm_flag = Ok;
 }
 
 /*---------------------------------------------------------------------------*/
 
-static void confirmCancelCb(UNUSED(Widget w), UNUSED(void *client_data),
-			    UNUSED(void *call_data))
-{
-  XtPopdown(shell);
-  confirm_flag = Cancel;
+static void confirmCancelCb(UNUSED(Widget w), void *client_data,
+			    UNUSED(void *call_data)) {
+	struct ConfirmationDialog *d = client_data;
+
+	XtPopdown(d->shell);
+	d->confirm_flag = Cancel;
 }
 
 /*---------------------------------------------------------------------------*/
 
-static void confirmAbortCb(UNUSED(Widget w), UNUSED(void *client_data),
-			   UNUSED(void *call_data))
-{
-  XtPopdown(shell);
-  confirm_flag = Abort;
+static void confirmAbortCb(UNUSED(Widget w), void *client_data,
+			   UNUSED(void *call_data)) {
+	struct ConfirmationDialog *d = client_data;
+
+	XtPopdown(d->shell);
+	d->confirm_flag = Abort;
 }
 
 /*-----------------------------------------------------------------------------
@@ -124,68 +130,88 @@ static const ButtonRec confirm_buttons[] = {
   PUBLIC FUNCTIONS
 -----------------------------------------------------------------------------*/
 
-void createConfirmPopup(void)
+static void createConfirmPopup(Widget parent, struct ConfirmationDialog **out)
 {
   Widget form, button_box;
+  struct ConfirmationDialog *d;
+
+  d = (struct ConfirmationDialog*)XtMalloc(sizeof(struct ConfirmationDialog));
 
   /* create shell */
-  shell = XtCreatePopupShell("confirm", transientShellWidgetClass,
-			     aw.shell, shell_args, XtNumber(shell_args));
+  shell_args[1].value = (XtArgVal) parent;
+  d->shell = XtCreatePopupShell("confirm", transientShellWidgetClass,
+			     parent, shell_args, XtNumber(shell_args));
 
   /* create outer form */
-  form = XtCreateManagedWidget("form", formWidgetClass, shell,
+  form = XtCreateManagedWidget("form", formWidgetClass, d->shell,
 				      form_args, XtNumber(form_args) );
 
   /* create label 1 */
   label_args[4].value = (XtArgVal) resources.label_font;
-  label1 = XtCreateManagedWidget("label1",labelWidgetClass, form,
+  d->label1 = XtCreateManagedWidget("label1",labelWidgetClass, form,
 				 label_args, XtNumber(label_args) );
 
   /* create label 2 */
-  label_args[1].value = (XtArgVal) label1;
-  label2 = XtCreateManagedWidget("label2",labelWidgetClass, form,
+  label_args[1].value = (XtArgVal) d->label1;
+  d->label2 = XtCreateManagedWidget("label2",labelWidgetClass, form,
 				 label_args, XtNumber(label_args) );
 
   /* create label 3 */
-  label_args[1].value = (XtArgVal) label2;
-  label3 = XtCreateManagedWidget("label3",labelWidgetClass, form,
+  label_args[1].value = (XtArgVal) d->label2;
+  d->label3 = XtCreateManagedWidget("label3",labelWidgetClass, form,
 				 label_args, XtNumber(label_args) );
 
   /* create button box */
-  button_box_args[1].value = (XtArgVal) label3;
+  button_box_args[1].value = (XtArgVal) d->label3;
   button_box = XtCreateManagedWidget("button box", boxWidgetClass, form,
-				     button_box_args, 
+				     button_box_args,
 				     XtNumber(button_box_args) );
-  createButtons(confirm_buttons, XtNumber(confirm_buttons), button_box,
-		NULL);
+  createButtons(confirm_buttons, XtNumber(confirm_buttons), button_box, d);
 
 #ifdef ENHANCE_POP_ACCEL
-  XtInstallAllAccelerators(form,form);
-  XtInstallAllAccelerators(shell,form);
+  XtInstallAllAccelerators(form, form);
+  XtInstallAllAccelerators(d->shell, form);
 #endif
 
-  XtRealizeWidget(shell);
-  setWMProps(shell);
+  XtRealizeWidget(d->shell);
+  setWMProps(d->shell);
+  *out = d;
 }
 
 /*---------------------------------------------------------------------------*/
 
-int confirm(const char *s1, const char *s2, const char *s3)
+static int confirm(struct ConfirmationDialog *d, const char *s1, const char *s2, const char *s3,
+		Boolean *aborted)
 {
   XEvent e;
 
-  XtVaSetValues(label1, XtNlabel, s1, NULL);
-  XtVaSetValues(label2, XtNlabel, s2, NULL);
-  XtVaSetValues(label3, XtNlabel, s3, NULL);
-  popupByCursor(shell, XtGrabExclusive);
+  XtVaSetValues(d->label1, XtNlabel, s1, NULL);
+  XtVaSetValues(d->label2, XtNlabel, s2, NULL);
+  XtVaSetValues(d->label3, XtNlabel, s3, NULL);
+  popupByCursor(d->shell, XtGrabExclusive);
 
-  confirm_flag = DontKnow;
+  d->confirm_flag = DontKnow;
 
   do {
     XtAppNextEvent(app_context, &e);
     XtDispatchEvent(&e);
-  } while (confirm_flag == DontKnow);
+  } while (d->confirm_flag == DontKnow);
 
-  aborted = (confirm_flag == Abort);
-  return (confirm_flag == Ok);
+  if (aborted != NULL)
+    *aborted = (d->confirm_flag == Abort);
+  return (d->confirm_flag == Ok);
+}
+
+int fwConfirm(FileWindowRec *fw, const char *s1, const char *s2, const char *s3, Boolean *aborted) {
+	if (fw->confirmationDialog == NULL)
+		createConfirmPopup(fw->shell, &fw->confirmationDialog);
+	return confirm(fw->confirmationDialog, s1, s2, s3, aborted);
+}
+
+static struct ConfirmationDialog *amConfirmDialog = NULL;
+
+int amConfirm(const char *s1, const char *s2, const char *s3, Boolean *aborted) {
+	if (amConfirmDialog == NULL)
+		createConfirmPopup(aw.shell, &amConfirmDialog);
+	return confirm(amConfirmDialog, s1, s2, s3, aborted);
 }
