@@ -1,7 +1,7 @@
 /*
  *  This file is part of X-File Manager XFM
  *  ----------------------------------------------------------------------
-  FmFw.c                                                             
+  FmFw.c
 
   (c) Simon Marlow 1991
   (c) Albert Graef 1994
@@ -11,11 +11,11 @@
 
   modified 7-1997 by strauman@sun6hft.ee.tu-berlin.de to add
   different enhancements (see README-1.4).
-                                                                           
-  modified 2005,2006 by Bernhard R. Link (see Changelog)
 
-  functions & data for creating a file window, and various functions        
-  related to file windows                                                   
+  modified 2005,2006,2007 by Bernhard R. Link (see Changelog)
+
+  functions & data for creating a file window, and various functions
+  related to file windows
  *  ----------------------------------------------------------------------
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -470,7 +470,7 @@ static void fileListCb(Widget w, XtPointer cld, XtPointer cad)
 {
  FileWindowRec  *fw=(FileWindowRec*)cld;
  String	        *args=(String*)cad;
- String	        entry_path,mess;
+ String	        entry_path;
  Widget 	entry;
 
  if ((entry_path=args[0])==0) {
@@ -480,10 +480,9 @@ static void fileListCb(Widget w, XtPointer cld, XtPointer cad)
  }
 
  if ((entry=XtNameToWidget(fw->shell,entry_path))==0) {
-	mess=XtMalloc(256);
-        sprintf(mess,"fileListCb (Notify): menu entry '%s' not found",entry_path);
-	XtAppWarning(XtWidgetToApplicationContext(w),mess);
-	XtFree(mess);
+	XtAppWarningFmt(XtWidgetToApplicationContext(w),
+        	"fileListCb (Notify): menu entry '%s' not found",
+		entry_path);
  	return;
  }
 
@@ -1040,6 +1039,47 @@ void initFileWindows(void)
   initFiles();
 }
 
+/*---------------------------------------------------------------------------*/
+
+static void fileDestroyed(UNUSED(Widget w), XtPointer client_data, UNUSED(XtPointer call_data))
+{
+  FileWindowRec *p, *fw = client_data;
+  int d;
+
+  if ((d = findDev(fw->directory)) != -1) umountDev(d);
+
+#ifdef ENHANCE_BUGFIX
+  /* The registered event handler should be removed */
+  XtRemoveEventHandler(fw->shell,XtAllEvents,True,
+	(XtEventHandler)clientMessageHandler,(XtPointer)0);
+#endif
+#ifdef ENHANCE_SELECTION
+  /* give the selection up now (might be called
+   * after destroying fw->shell!
+   */
+  FmDisownSelection(fw);
+#endif
+
+  if (fw == file_windows)
+    file_windows = fw->next;
+  else {
+    for (p = file_windows; p != NULL && p->next != fw; p = p->next);
+    if( p != NULL )
+      p->next = fw->next;
+  }
+
+  fw->shell = None;
+
+  freeFileList(fw);
+  XTFREE(fw->file_items);
+  XTFREE(fw->folder_items);
+  XTFREE(fw->view_items);
+  XTFREE(fw->option_items);
+  XTFREE(fw);
+
+  chdir(user.home);
+}
+
 /*---------------------------------------------------------------------------*/
 /* Create a file Window at the specified path, in the specified format */
 
@@ -1059,7 +1099,7 @@ static FileWindowRec *createFileWindow(const char *path, const char *title,
   }
 
   /* put at front of linked list */
-  fw = (FileWindowRec *) XtMalloc(sizeof(FileWindowRec));
+  fw = (FileWindowRec *) XtCalloc(1, sizeof(FileWindowRec));
   fw->next = file_windows;
   file_windows = fw;
   
@@ -1162,6 +1202,8 @@ static FileWindowRec *createFileWindow(const char *path, const char *title,
   fprintf(stderr, "exiting createFileWindow: %lu\n", malloc_inuse(NULL));
 #endif
 
+  XtAddCallback(fw->shell, XtNdestroyCallback, fileDestroyed, fw);
+
   return fw;
 }
 
@@ -1416,7 +1458,7 @@ void reDisplayFileWindow(FileWindowRec *fw)
   Use markForUpdate() to explicitly mark a directory for update.
   Call intUpdate to execute all the actions.
 -----------------------------------------------------------------------------*/
-void markForUpdate(String path)
+void markForUpdate(const char *path)
 {
   FileWindowRec *fw;
 
@@ -1499,8 +1541,8 @@ void intUpdate(void)
 -----------------------------------------------------------------------------*/
 void updateStatus(FileWindowRec *fw)
 {
-  char s[1024], t[1024];
-  int n_files, n_selections;
+  char *line;
+  int r, n_files, n_selections;
   long n_bytes, n_bytes_selected;
 
   if (fw->n_selections >= 1) {
@@ -1594,23 +1636,30 @@ void updateStatus(FileWindowRec *fw)
     n_files--;
   }
 
-  if (fw->do_filter)
-    sprintf(t, " [%s]", fw->dirFilter);
-  else
-    *t = '\0';
-
   if (n_selections > 0)
-    sprintf(s,
-	"%ld byte%s in %d item%s, %ld byte%s in %d selected item%s%s",
+    r = asprintf(&line,
+	"%ld byte%s in %d item%s, %ld byte%s in %d selected item%s%s%s%s",
 	n_bytes, n_bytes==1?"":"s",
 	n_files, n_files==1?"":"s",
 	n_bytes_selected, n_bytes_selected==1?"":"s",
-	n_selections, n_selections==1?"":"s", t);
+	n_selections, n_selections==1?"":"s",
+	fw->do_filter?" [":"",
+	fw->do_filter?fw->dirFilter:"",
+	fw->do_filter?"]":"");
   else
-    sprintf(s, "%ld byte%s in %d item%s%s", n_bytes, n_bytes==1?"":"s",
-	n_files, n_files==1?"":"s", t);
+    r = asprintf(&line,
+	"%ld byte%s in %d item%s%s%s%s", n_bytes, n_bytes==1?"":"s",
+	n_files, n_files==1?"":"s",
+	fw->do_filter?" [":"",
+	fw->do_filter?fw->dirFilter:"",
+	fw->do_filter?"]":"");
 
-  XtVaSetValues(fw->status, XtNlabel, (XtArgVal) s, NULL);
+  if( r < 0 || line == NULL )
+    XtVaSetValues(fw->status, XtNlabel, (XtArgVal) "Error", NULL);
+  else {
+    XtVaSetValues(fw->status, XtNlabel, (XtArgVal) line, NULL);
+    free(line);
+  }
 }
 
 /*----------------------------------------------------------------------------
