@@ -9,7 +9,7 @@
   modified 7-1997 by strauman@sun6hft.ee.tu-berlin.de to add
   different enhancements (see README-1.4).
 
-  modified 2004,2005,2006 by Bernhard R. Link (see Changelog)
+  modified 2004,2005,2006,2007 by Bernhard R. Link (see Changelog)
 
   Action procedures for widgets in a file window
  *  ----------------------------------------------------------------------
@@ -65,9 +65,6 @@ Boolean dragging = False;
 static Cursor drag_cursor;
 static Boolean was_dropable;
 
-static char from[MAXPATHLEN], to[MAXPATHLEN];
-static int fromi, toi;
-
 /*---------------------------------------------------------------------------
   PRIVATE FUNCTIONS
 ---------------------------------------------------------------------------*/
@@ -97,19 +94,11 @@ static int findWidget(Widget w, FileWindowRec **fw_ret, XEvent *ev)
 
 /*---------------------------------------------------------------------------*/
 
-static int moveFiles(void)
+static int moveFiles(FileWindowRec *fw, const char *fromdir, const char *todir)
 {
   int i, n_moved = 0;
-
-  if (from[fromi-1] != '/') {
-    from[fromi++] = '/';
-    from[fromi] = '\0';
-  }
-
-  if (to[toi-1] != '/') {
-    to[toi++] = '/';
-    to[toi] = '\0';
-  }
+  char *from, *to;
+  Boolean aborted;
 
   for (i=0; i < move_info.fw->n_files; i++)
     if (move_info.fw->files[i]->selected) {
@@ -118,13 +107,13 @@ static int moveFiles(void)
 	error("Cannot move . or ..", "");
 	continue;
       }
-      strcpy(from+fromi, move_info.fw->files[i]->name);
-      strcpy(to+toi, move_info.fw->files[i]->name);
+      from = dirConcat(fromdir, move_info.fw->files[i]->name);
+      to = dirConcat(todir, move_info.fw->files[i]->name);
       if (exists(to) && resources.confirm_overwrite) {
-	char s[0xff];
-	sprintf(s, "Move: file %s already exists at destination",
-		move_info.fw->files[i]->name);
-	if (!confirm(s, "Overwrite?", "")) {
+	if (!fwConfirm(fw, "Move: file already exists at destination", 
+		     move_info.fw->files[i]->name, "Overwrite?", &aborted)) {
+	  XTFREE(from);
+	  XTFREE(to);
 	  if (aborted)
 	    break;
 	  else
@@ -132,11 +121,11 @@ static int moveFiles(void)
 	}
       }
       if (rmove(from,to)) {
-	char s[0xff];
-	sprintf(s, "Error moving %s:", move_info.fw->files[i]->name);
-	sysError(s);
+	sysErrorFmt("Error moving %s:", move_info.fw->files[i]->name);
       } else
 	n_moved++;
+      XTFREE(from);
+      XTFREE(to);
     }
 
   return n_moved;
@@ -144,19 +133,11 @@ static int moveFiles(void)
 
 /*---------------------------------------------------------------------------*/
 
-static int copyFiles(void)
+static int copyFiles(FileWindowRec *fw, const char *fromdir, const char *todir)
 {
   int i, n_copied = 0;
-
-  if (from[fromi-1] != '/') {
-    from[fromi++] = '/';
-    from[fromi] = '\0';
-  }
-
-  if (to[toi-1] != '/') {
-    to[toi++] = '/';
-    to[toi] = '\0';
-  }
+  char *from, *to;
+  Boolean aborted;
 
   for (i=0; i < move_info.fw->n_files; i++)
     if (move_info.fw->files[i]->selected) {
@@ -165,13 +146,13 @@ static int copyFiles(void)
 	error("Cannot copy . or ..", "");
 	continue;
       }
-      strcpy(from+fromi, move_info.fw->files[i]->name);
-      strcpy(to+toi, move_info.fw->files[i]->name);
+      from = dirConcat(fromdir, move_info.fw->files[i]->name);
+      to = dirConcat(todir, move_info.fw->files[i]->name);
       if (exists(to) && resources.confirm_overwrite) {
-	char s[0xff];
-	sprintf(s, "Copy: file %s already exists at destination",
-		move_info.fw->files[i]->name);
-	if (!confirm(s, "Overwrite?", "")) {
+	if (!fwConfirm(fw, "Copy: file already exists at destination",
+		move_info.fw->files[i]->name, "Overwrite?", &aborted)) {
+	  XTFREE(from);
+	  XTFREE(to);
 	  if (aborted)
 	    break;
 	  else
@@ -179,11 +160,11 @@ static int copyFiles(void)
 	}
       }
       if (rcopy(from,to)) {
-	char s[0xff];
-	sprintf(s, "Error copying %s:", move_info.fw->files[i]->name);
-	sysError(s);
+	sysErrorFmt("Error copying %s:", move_info.fw->files[i]->name);
       } else
 	n_copied++;
+      XTFREE(from);
+      XTFREE(to);
     }
 
   return n_copied;
@@ -191,21 +172,20 @@ static int copyFiles(void)
 
 /*---------------------------------------------------------------------------*/
 
-static int setupMoveCopy(FileWindowRec *fw, int i)
+static int setupMoveCopy(FileWindowRec *fw, int i, char **todir_buf, const char **todir)
 {
-  strcpy(from, move_info.fw->directory);
+  *todir_buf = NULL;
 
   if (i != -1 && S_ISDIR(fw->files[i]->stats.st_mode)
       && strcmp(fw->files[i]->name, ".")) {
-    if (chdir(fw->directory) || chdir(fw->files[i]->name) || !getwd(to)) {
+    if (chdir(fw->directory) || chdir(fw->files[i]->name) || 
+		(*todir_buf = get_current_dir_name()) == NULL) {
       sysError("System error:");
       return 0;
     }
+    *todir = *todir_buf;
   } else
-    strcpy(to, fw->directory);
-    
-  toi = strlen(to);
-  fromi = strlen(from);
+    *todir = fw->directory;
 
   return 1;
 }
@@ -214,19 +194,21 @@ static int setupMoveCopy(FileWindowRec *fw, int i)
 
 static void fileEndMove(FileWindowRec *fw, int i)
 {
-  char fromdir[MAXPATHLEN], todir[MAXPATHLEN];
+  const char *destdir, *srcdir;
+  char *buffer;
 
-  if (!setupMoveCopy(fw, i)) return;
-  strcpy(fromdir, from);
-  strcpy(todir, to);
+  srcdir = move_info.fw->directory;
+  if (!setupMoveCopy(fw, i, &buffer, &destdir)) return;
 
-  if (!strcmp(from, to)) {
+  if (cmpCanonical(srcdir, destdir)) {
     error("Move:", "Source and destination are identical");
+    XTFREE(buffer);
     return;
   }
 
-  if (access(to, W_OK)) {
+  if (access(destdir, W_OK)) {
     error("No write access to this directory","");
+    XTFREE(buffer);
     return;
   }
 
@@ -236,18 +218,19 @@ static void fileEndMove(FileWindowRec *fw, int i)
     char s1[0xff], s2[0xff], s3[0xff];
     sprintf(s1, "Moving %d item%c", move_info.fw->n_selections,
 	    move_info.fw->n_selections > 1 ? 's' : ' ');
-    sprintf(s2, "from: %s", from);
-    sprintf(s3, "to: %s", to);
-    if (!confirm(s1, s2, s3))
+    sprintf(s2, "from: %s", srcdir);
+    sprintf(s3, "to: %s", destdir);
+    if (!fwConfirm(fw, s1, s2, s3, NULL))
       goto out;
   }
 
-  if (moveFiles()) {
-    markForUpdate(fromdir); markForUpdate(todir);
+  if (moveFiles(fw, srcdir, destdir)) {
+    markForUpdate(srcdir); markForUpdate(destdir);
     intUpdate();
   }
 
  out:
+  XTFREE(buffer);
   freeze = False;
 }
 
@@ -255,18 +238,21 @@ static void fileEndMove(FileWindowRec *fw, int i)
 
 static void fileEndCopy(FileWindowRec *fw, int i)
 {
-  char todir[MAXPATHLEN];
+  char *buffer;
+  const char *srcdir, *destdir;
 
-  if (!setupMoveCopy(fw, i)) return;
-  strcpy(todir, to);
+  srcdir = move_info.fw->directory;
+  if (!setupMoveCopy(fw, i, &buffer, &destdir)) return;
 
-  if (!strcmp(from, to)) {
+  if (cmpCanonical(srcdir, destdir)) {
     error("Copy:", "Source and destination are identical");
+    XTFREE(buffer);
     return;
   }
 
-  if (access(to, W_OK)) {
+  if (access(destdir, W_OK)) {
     error("No write access to this directory","");
+    XTFREE(buffer);
     return;
   }
 
@@ -276,18 +262,19 @@ static void fileEndCopy(FileWindowRec *fw, int i)
     char s1[0xff], s2[0xff], s3[0xff];
     sprintf(s1, "Copying %d item%c", move_info.fw->n_selections,
 	    move_info.fw->n_selections > 1 ? 's' : ' ');
-    sprintf(s2, "from: %s", from);
-    sprintf(s3, "to: %s", to);
-    if (!confirm(s1, s2, s3))
+    sprintf(s2, "from: %s", srcdir);
+    sprintf(s3, "to: %s", destdir);
+    if (!fwConfirm(fw, s1, s2, s3, NULL))
       goto out;
   }
 
-  if (copyFiles()) {
-    markForUpdate(todir);
+  if (copyFiles(fw, srcdir, destdir)) {
+    markForUpdate(destdir);
     intUpdate();
   }
 
  out:
+  XTFREE(buffer);
   freeze = False;
 }
 
