@@ -4,7 +4,7 @@
  *  Enhancements to the X-File Manager XFM-1.3.2 (Path History)
  *  Copyright (C) 1997  by Till Straumann   <strauman@sun6hft.ee.tu-berlin.de>
  *
- *  modified 2004,2005,2006 by Bernhard R. Link (see Changelog)
+ *  modified 2004,2005,2006,2007 by Bernhard R. Link (see Changelog)
  *  ----------------------------------------------------------------------
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -143,7 +143,7 @@ HistoryList FmCreateHistoryList(const char* name, Widget parent, char *fixed_pat
  return hl;
 }
 
-void FmInsertHistoryPath(HistoryList hl, char *path)
+void FmInsertHistoryPath(HistoryList hl, const char *path)
 {
  Widget   *chlds;
  Widget	  history_menu=hl->menu;
@@ -205,7 +205,7 @@ void FmInsertHistoryPath(HistoryList hl, char *path)
 }
 	
 
-void FmDeleteHistoryPath(HistoryList hl, char *path)
+void FmDeleteHistoryPath(HistoryList hl, const char *path)
 {
  Arg arg;
  Widget   *chlds,found;
@@ -341,22 +341,20 @@ action called before popping up the menu?");
 static void HistoryCB(Widget w, XtPointer cld, UNUSED(XtPointer cad))
 {
   char 		path[MAXPATHLEN];
-  char 		*str;
+  char *dir;
+  const char 		*str;
   HistoryList	hl=(HistoryList)cld;
-  char 		s[0xff];
   Widget	p;
 #ifdef ENHANCE_SCROLL
   Boolean	keep_position=True;
 #endif
 
   XtVaGetValues(w, XtNlabel, &str, (char*)0);
-  strcpy(path,str);
-
-  fnexpand(path);
+  dir = dyn_fnexpand(str);
 
   /* was the history list set up correctly ? */
   if (hl==0 || hl->menu != XtParent(w)) {
-	cb_error(); return;
+	cb_error(); XtFree(dir); return;
   }
 
   /* if the widget that popped up the menu is a text field,
@@ -369,9 +367,10 @@ static void HistoryCB(Widget w, XtPointer cld, UNUSED(XtPointer cad))
 	/* Suppress parsing of the string for special character
 	 * sequences (\t etc), take it 'as is'
 	 */
-	args[0]=path;
+	args[0]=dir;
 	args[1]="F"; 
 	XtCallActionProc(hl->emanate,"InsertChar",0,(char **)args,2);
+	XtFree(dir);
 	return;
   }
 
@@ -381,18 +380,17 @@ static void HistoryCB(Widget w, XtPointer cld, UNUSED(XtPointer cad))
   while(p && p != hl->fw->shell) p=XtParent(p);
   if (!p) {
 	cb_error();
+	XtFree(dir);
 	return;
   }
   
 
-  if (chdir(str=path)) {
-    sprintf(s, "Can't open folder %s:", path);
-    sysError(s);
-    FmDeleteHistoryPath(hl,path);
+  if (chdir(str=dir)) {
+    sysErrorFmt("Can't open folder %s:", dir);
+    FmDeleteHistoryPath(hl, dir);
     if (chdir(str=hl->fw->directory)) {
-    	sprintf(s, "Can't open folder %s:", str);
-        sysError(s);
-        FmDeleteHistoryPath(hl,str);
+        sysErrorFmt("Can't open folder %s:", str);
+        FmDeleteHistoryPath(hl, str);
     }
   } else if (!getwd(path))
     sysError("System error:");
@@ -407,6 +405,7 @@ static void HistoryCB(Widget w, XtPointer cld, UNUSED(XtPointer cad))
     updateFileDisplay(hl->fw);
 #endif
   }
+  XtFree(dir);
 }
 
 
@@ -416,7 +415,6 @@ Widget 		parent;
 FileWindowRec	*fw;
 HistoryList	hl;
 String		str;
-char 		error_buf[BUFSIZ];
 WidgetList	chlds,destroy;
 int		i,line_i;
 Cardinal	num_children;
@@ -425,10 +423,7 @@ Arg		arg;
 struct stat 	buf; 
 
 if (*nargs != 1) {
-    (void) sprintf(error_buf, "%s %s",
-	    "UpdateHistory:  expects only one",
-	    "parameter which is the name of the menu.");
-    XtAppWarning(XtWidgetToApplicationContext(w), error_buf);
+    XtAppWarning(XtWidgetToApplicationContext(w), "UpdateHistory:  expects only one parameter which is the name of the menu.");
     return;
 }
 
@@ -437,10 +432,9 @@ hl=histories;
 while(hl && strcmp(XtName(hl->menu),args[0])) hl=hl->next;
 
 if (!hl) {
-    (void) sprintf(error_buf, "%s '%s' %s",
-	    "UpdateHistory:  menu",args[0],
-	    "not found in history list");
-    XtAppWarning(XtWidgetToApplicationContext(w), error_buf);
+    XtAppWarningFmt(XtWidgetToApplicationContext(w),
+		    "UpdateHistory:  menu '%s' not found in history list",
+		    args[0]);
     return;
 }
 
