@@ -4,9 +4,10 @@
 /*
  *  Enhancements to the X-File Manager XFM-1.3.2 (Path History)
  *  -----------------------------------------------------------
- 
  *  Copyright (C) 1997  by Till Straumann   <strauman@sun6hft.ee.tu-berlin.de>
-
+ *
+ *  modified 2007 by Bernhard R. Link (see Changelog)
+ *  -----------------------------------------------------------
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -42,10 +43,10 @@ typedef struct HistoryList_s *HistoryList;
 HistoryList FmCreateHistoryList(const char* name, Widget parent, char *fixed_paths[]);
 
 /* insert path on top */
-void FmInsertHistoryPath(HistoryList hl, char *path);
+void FmInsertHistoryPath(HistoryList hl, const char *path);
 
 /* delete entry (no action if path not found in list) */
-void FmDeleteHistoryPath(HistoryList hl, char *path);
+void FmDeleteHistoryPath(HistoryList hl, const char *path);
 
 /* chop the list to n entries */
 void FmChopHistoryList(HistoryList hl, int n);
