@@ -1,3 +1,31 @@
+/*
+ *  This file is part of X-File Manager XFM
+ *  ----------------------------------------------------------------------
+  FmLog.c
+
+  Copyright (C) 1997 by T. Straumann, <strauman@sun6hft.ee.tu-berlin.de>
+
+  modified 2006 by Bernhard R. Link (see Changelog)
+
+  A logging window has been added to show output of child processes sent 
+  to stdout / stderr.
+ *  ----------------------------------------------------------------------
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+#include <xfmconfig.h>
+
 #include <stdio.h>
 #include <errno.h>
 #include <unistd.h>
@@ -7,15 +35,18 @@
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
 
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Viewport.h>
-#include <X11/Xaw/Label.h>
-#include <X11/Xaw/Command.h>
-#include <X11/Xaw/Toggle.h>
-#include <X11/Xaw/AsciiText.h>
+#include <X11/Xaw3d/Form.h>
+#include <X11/Xaw3d/Viewport.h>
+#include <X11/Xaw3d/Label.h>
+#include <X11/Xaw3d/Command.h>
+#include <X11/Xaw3d/Toggle.h>
+#include <X11/Xaw3d/AsciiText.h>
+
+#include "global.h"
 
 #include "Fm.h"
 #include "FmLog.h"
+#include "execute.h"
 
 typedef struct LogData_ {
 	Widget shell,text,auto_flag;
@@ -24,13 +55,11 @@ typedef struct LogData_ {
 	
 
 Widget		fmLogPopupShell=0;
-FILE	*orig_stderr;
-static FILE	*orig_stdout;
 static int	peipd[2];
 
 static XawTextPosition pos=(XawTextPosition)0;
 
-static void get_pipe_input(XtPointer cld,int *fid, XtInputId *id)
+static void get_pipe_input(XtPointer cld, int *fid, UNUSED(XtInputId *id))
 {
  XawTextBlock           block;
  int  			nbytes;
@@ -39,10 +68,10 @@ static void get_pipe_input(XtPointer cld,int *fid, XtInputId *id)
  LogData		*ldp=(LogData*)cld;
 
  if ((nbytes=read(*fid,buf,BUFSIZ))==-1)
-	fprintf(orig_stderr,"\nread error %i in 'get_pipe_input()'\n",errno);
+	fprintf(stderr,"\nread error %i in 'get_pipe_input()'\n",errno);
 
  if (nbytes>0) {
-   XtVaSetValues(ldp->text,XtNeditType,XawtextEdit,0);
+   XtVaSetValues(ldp->text,XtNeditType,XawtextEdit,(char*)0);
    block.format=FMT8BIT;
    block.length=nbytes;
    block.ptr=buf;
@@ -56,43 +85,44 @@ static void get_pipe_input(XtPointer cld,int *fid, XtInputId *id)
    XtVaSetValues(ldp->text,
 		XtNeditType,XawtextRead,
 		XtNinsertPosition,pos,
-		0);
+		(char*)0);
 
-   XtVaGetValues(ldp->auto_flag,XtNstate,&auto_popup,0);
+   XtVaGetValues(ldp->auto_flag,XtNstate,&auto_popup,(char*)0);
    if (auto_popup)
      popupByCursor(ldp->shell,XtGrabNone);
  }
 
 }
 
-static void update_tick_cb(Widget w, XtPointer cld, XtPointer cad)
+static void update_tick_cb(Widget w, XtPointer cld, UNUSED(XtPointer cad))
 {
 Boolean flag;
 Pixel   bg=(Pixel)cld,fg;
-XtVaGetValues(w,XtNstate,&flag,XtNforeground,&fg,0);
-XtVaSetValues(w,XtNbackground,(flag?bg:fg),0);
+XtVaGetValues(w,XtNstate,&flag,XtNforeground,&fg,(char*)0);
+XtVaSetValues(w,XtNbackground,(flag?bg:fg),(char*)0);
 }
 
-static void hide_log_cb(Widget w, XtPointer cld, XtPointer cad)
+static void hide_log_cb(UNUSED(Widget w), XtPointer cld, UNUSED(XtPointer cad))
 {
  Widget shell=(Widget)cld;
  XtPopdown(shell);
 }
  
-static void clear_log_cb(Widget w, XtPointer cld, XtPointer cad)
+static void clear_log_cb(UNUSED(Widget w), XtPointer cld, UNUSED(XtPointer cad))
 {
  Widget text=(Widget)cld;
  XawTextBlock           block;
 
- XtVaSetValues(text,XtNeditType,XawtextEdit,0);
+ XtVaSetValues(text,XtNeditType,XawtextEdit,(char*)0);
  block.firstPos=0; block.length=0; block.ptr=0;
  block.format=FMT8BIT;
  XawTextReplace(text,0,pos,&block);
- XtVaSetValues(text,XtNeditType,XawtextRead,0);
+ XtVaSetValues(text,XtNeditType,XawtextRead,(char*)0);
  pos=0;
 }
 
-static void destroy_log_cb(Widget w, XtPointer cld, XtPointer cad)
+static void destroy_log_cb(UNUSED(Widget w), XtPointer cld,
+                           UNUSED(XtPointer cad))
 {
 LogData *ldp=(LogData*)cld;
 
@@ -100,11 +130,10 @@ XtRemoveInput(ldp->inp_id);
 /* try to restore stdout/stderr, hope the buffering doesn't
  * get mixed up :-0
  */
-fflush(stdout); fflush(stderr);
-close(STDOUT_FILENO); dup(fileno(orig_stdout)); fclose(orig_stdout);
-close(STDERR_FILENO); dup(fileno(orig_stderr)); fclose(orig_stderr);
+child_stdout = -1; child_stderr = -1;
+/* let's hope there is not child left to choke on its stdout vanishing */
 close(peipd[0]);
-
+close(peipd[1]);
 fmLogPopupShell=0;
 XtFree((char*)ldp);
 }
@@ -149,7 +178,7 @@ Widget FmCreateLog(Widget parent,XFontStruct *font)
 	  "form", formWidgetClass, ldp->shell, 
 	  NULL);
 
-  XtVaGetValues(form,XtNdefaultDistance,&vdist,0);
+  XtVaGetValues(form,XtNdefaultDistance,&vdist,(char*)0);
 
 
   button_args[FONT].value=(XtArgVal)font;
@@ -184,7 +213,7 @@ Widget FmCreateLog(Widget parent,XFontStruct *font)
 	  button_args,N_BUTTON_ARGS);
   XtAddCallback(cmd,XtNcallback,clear_log_cb,(XtPointer)ldp->text);
 
-  XtVaGetValues(cmd,XtNheight,&h1, XtNborderWidth,&bw, 0);
+  XtVaGetValues(cmd,XtNheight,&h1, XtNborderWidth,&bw, (char*)0);
 
   button_args[FROM_HORIZ].value=(XtArgVal)cmd;
   cmd =XtVaCreateManagedWidget("Auto Popup", labelWidgetClass, form,
@@ -197,7 +226,7 @@ Widget FmCreateLog(Widget parent,XFontStruct *font)
 	  XtNbottom,XtChainTop,
 	  XtNfont,font,
 	  XtNheight,h1+2*bw,
-	  0);
+	  (char*)0);
 
   button_args[FROM_HORIZ].value=(XtArgVal)cmd;
   ldp->auto_flag =XtCreateManagedWidget(
@@ -208,7 +237,7 @@ Widget FmCreateLog(Widget parent,XFontStruct *font)
   XtVaGetValues(ldp->auto_flag,
 	XtNforeground,&fg,XtNbackground,&bg,XtNstate,&flag,
 	XtNheight,&h2,
-	0);
+	(char*)0);
 
   vdist+=(h1-h2)/2;
 
@@ -216,18 +245,13 @@ Widget FmCreateLog(Widget parent,XFontStruct *font)
 	XtNvertDistance,vdist,
 	XtNforeground,bg,
 	XtNbackground,(flag?fg:bg),
-	0);
+	(char*)0);
 
   XtAddCallback(ldp->auto_flag,XtNcallback,update_tick_cb,(XtPointer)fg);
 	
   pipe(peipd);
-  orig_stderr=fdopen(dup(STDERR_FILENO),"w");
-  orig_stdout=fdopen(dup(STDOUT_FILENO),"w");
-  /* close old stderr, stdout */
-  close(STDOUT_FILENO); close(STDERR_FILENO);
-  /* redirect stdout, stderr to the pipe */
-  dup(peipd[1]); dup(peipd[1]);
-  close(peipd[1]);
+  child_stderr = peipd[1];
+  child_stdout = peipd[1];
   ldp->inp_id=XtAppAddInput(
 	app,
 	peipd[0],
@@ -248,7 +272,8 @@ Widget FmCreateLog(Widget parent,XFontStruct *font)
   return ldp->shell;
 }
 
-void logPopup(Widget w, FileWindowRec *fw, XtPointer cad)
+void logPopup(UNUSED(Widget w), UNUSED(FileWindowRec *fw),
+              UNUSED(XtPointer cad))
 {
 if (fmLogPopupShell) 
  popupByCursor(fmLogPopupShell,XtGrabNone);
