@@ -237,7 +237,7 @@ static void renameOkCb(UNUSED(Widget w), struct PopupDialog *p,
     } else {
       char *todir = dir_prefix(to);
       markForUpdate(fw->directory); markForUpdate(todir);
-      XTFREE(to);
+      XTFREE(todir);
       intUpdate();
     }
 
