@@ -1,7 +1,7 @@
 #ifndef FmSelection_h
 #define FmSelection_h
 
-Boolean FmOwnSelection(FileWindowRec *fw, Time time);
+Boolean FmOwnSelection(FileWindowRec *fw, Time t);
 
 void FmDisownSelection(FileWindowRec *fw);
 void FmSelectionInit(Display *di);
