@@ -12,7 +12,7 @@
   modified 7-1997 by strauman@sun6hft.ee.tu-berlin.de to add
   different enhancements (see README-1.4).
 
-  modified 2004,2005,2006 by Bernhard R. Link (see Changelog)
+  modified 2004,2005,2006,2007 by Bernhard R. Link (see Changelog)
 
   General utility functions for creating menus, buttons, questions,
   and functions for desensetising and 'ticking' menu entries.
@@ -69,7 +69,8 @@
 -----------------------------------------------------------------------------*/
 
 static Arg shell_args[] = {
-  { XtNtitle, (XtArgVal) NULL }
+  { XtNtitle, (XtArgVal) NULL },
+  { XtNtransientFor, (XtArgVal) NULL }
 };
 
 static Arg form_args[] = {
@@ -285,15 +286,6 @@ void popdownPopup(struct PopupDialog *p)
 {
 	XtPopdown(p->shell);
 }
-void destroyPopup(struct PopupDialog *p)
-{
-	Cardinal i;
-	for (i=0; i < p->count ; i++) {
-		XTFREE(p->fields[i].answer);
-	}
-	XtDestroyWidget(p->shell);
-	XTFREE(p);
-}
 
 void reloadPopupData(struct PopupDialog *p)
 {
@@ -338,9 +330,24 @@ void popupPopupByCursor(struct PopupDialog *p, void *data)
 	popupByCursor(p->shell, XtGrabExclusive);
 }
 
+static void popupDialogDestroyed(UNUSED(Widget w), XtPointer client_data, UNUSED(XtPointer call_data)) {
+	Cardinal i;
+	struct PopupDialog *p = client_data;
+
+	p->shell = NULL;
+	if( p->clear_ptr != NULL )
+		*p->clear_ptr = NULL;
+	for (i=0; i < p->count ; i++) {
+		XTFREE(p->fields[i].answer);
+	}
+	XTFREE(p);
+}
+
 /*****************************************************************************/
 /* Function: createPopupQuestions                                            */
-/* Arguments: name        :  The widget name for the shell                   */
+/* Arguments: out         :  Variable to set and remember for clearing       */
+/*            parent      :  The widget's parent                             */
+/*            name        :  The widget name for the shell                   */
 /*            title       :  The title of the popup window                   */
 /*            bitmap      :  A bitmap to display to the left of the box      */
 /*            questions   :  A list of questions to use                      */
@@ -356,7 +363,8 @@ void popupPopupByCursor(struct PopupDialog *p, void *data)
 /*****************************************************************************/
 
 
-struct PopupDialog *createPopupQuestions(const char *name, const char *title,
+void createPopupQuestions(struct PopupDialog **out, Widget parent,
+                            const char *name, const char *title,
                             Pixmap bitmap,
 			    const QuestionRec *questions, Cardinal n_questions,
 			    const PDButtonRec *buttons, Cardinal n_buttons,
@@ -374,6 +382,7 @@ struct PopupDialog *createPopupQuestions(const char *name, const char *title,
 #endif
 #endif
   popup = (void*)XtMalloc(sizeof(struct PopupDialog)+n_questions*sizeof(struct PopupDialogField));
+  popup->clear_ptr = NULL;
   popup->questions = questions;
   popup->count = n_questions;
   for (i = 0; i < n_questions ; i++) {
@@ -381,7 +390,8 @@ struct PopupDialog *createPopupQuestions(const char *name, const char *title,
   }
   /* create popup shell */
   shell_args[0].value = (XtArgVal) title;
-  popup->shell = XtCreatePopupShell(name, transientShellWidgetClass, aw.shell,
+  shell_args[1].value = (XtArgVal) parent;
+  popup->shell = XtCreatePopupShell(name, transientShellWidgetClass, parent,
 			     shell_args, XtNumber(shell_args) );
 
   /* create form */
@@ -464,11 +474,13 @@ struct PopupDialog *createPopupQuestions(const char *name, const char *title,
 #endif
     XTFREE(button_widgets);
   }
+  popup->clear_ptr = out;
+  XtAddCallback(popup->shell, XtNdestroyCallback, popupDialogDestroyed, popup);
 
   XtRealizeWidget(popup->shell);
   setWMProps(popup->shell);
 
-  return popup;
+  *out = popup;
 }
 
 /*****************************************************************************/
@@ -628,7 +640,7 @@ static const PDButtonRec dialog_buttons[] = {
   { "cancel", "Cancel", dialogCancelCb }
 };
 
-char *varPopup(Pixmap icon_bm, char *action)
+char *varPopup(Widget parent, Pixmap icon_bm, char *action)
 {
   char *act = NULL;
   char *act1 = (char *)alloca(strlen(action)+1), *s, *t;
@@ -672,10 +684,13 @@ char *varPopup(Pixmap icon_bm, char *action)
       dialog_questions[i].length = MAXVARSTRINGLEN;
     }
 
-    dialog = createPopupQuestions("dialog", "Parameter Dialog", icon_bm,
+    createPopupQuestions(&dialog, parent,
+		                 "dialog", "Parameter Dialog", icon_bm,
 				  dialog_questions, n_vars, dialog_buttons,
 				  XtNumber(dialog_buttons),
 				  XtNumber(dialog_buttons)-1);
+    /* local variable, do not write to after this function is gone: */
+    dialog->clear_ptr = NULL;
     for (i = 0; i < n_vars; i++) {
 	    setPopupData(dialog, i, default_vals[i]);
     }
@@ -703,7 +718,7 @@ char *varPopup(Pixmap icon_bm, char *action)
     for (i = 0; i < n_vars; i++)
       XTFREE(vars[i]);
     XTFREE(acts); XTFREE(vars); XTFREE(default_vals);
-    destroyPopup(dialog);
+    XtDestroyWidget(dialog->shell);
     return act;
   } else {
     if (n_acts) XTFREE(acts);
@@ -716,7 +731,7 @@ char *varPopup(Pixmap icon_bm, char *action)
 /* variant of createPopupQuestions which shows some lines of left-justified
    text below a pixmap */
 
-static Widget createPopupText(String name, String title, Pixmap bitmap,
+static Widget createPopupText(Widget parent, String name, String title, Pixmap bitmap,
 			      const char **lines, Cardinal n_lines,
 			      const ButtonRec *buttons, Cardinal n_buttons,
 			      Cardinal dflt_button)
@@ -733,7 +748,8 @@ static Widget createPopupText(String name, String title, Pixmap bitmap,
 #endif
   /* create popup shell */
   shell_args[0].value = (XtArgVal) title;
-  shell = XtCreatePopupShell(name, transientShellWidgetClass, aw.shell,
+  shell_args[1].value = (XtArgVal) parent;
+  shell = XtCreatePopupShell(name, transientShellWidgetClass, parent,
 			     shell_args, XtNumber(shell_args) );
 
   /* create form */
@@ -774,7 +790,7 @@ static Widget createPopupText(String name, String title, Pixmap bitmap,
 #ifdef ENHANCE_POP_ACCEL
     button_widgets =
 #endif
-    createButtons(buttons, n_buttons, box_widget, NULL);
+    createButtons(buttons, n_buttons, box_widget, shell);
 
 #ifdef ENHANCE_POP_ACCEL
     XtInstallAllAccelerators(form_widget,form_widget);
@@ -810,12 +826,12 @@ static Widget createPopupText(String name, String title, Pixmap bitmap,
 
 /*---------------------------------------------------------------------------*/
 
-static Widget about = None;
-
-static void aboutOkCb(UNUSED(Widget w), UNUSED(XtPointer client_data), 
+static void aboutOkCb(UNUSED(Widget w), XtPointer client_data,
 		      UNUSED(XtPointer call_data))
 {
-  XtPopdown(about);
+  Widget about = client_data;
+
+  XtDestroyWidget(about);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -832,16 +848,15 @@ static const char *about_text[] = {
 
 #include "FmVersion.h"
 
-void aboutPopup(void)
+void aboutPopup(Widget parent)
 {
+  Widget about = None;
   Pixmap logo_bm;
-  if (about == None) {
-    logo_bm = loadIcon("XfmLogo.xpm", NULL, NULL);
-    about = createPopupText("about", "About xfm", logo_bm,
-			    about_text, XtNumber(about_text), about_buttons,
-			    XtNumber(about_buttons),
-			    XtNumber(about_buttons)-1);
-  }
+  logo_bm = loadIcon("XfmLogo.xpm", NULL, NULL);
+  about = createPopupText(parent, "about", "About xfm", logo_bm,
+		  about_text, XtNumber(about_text), about_buttons,
+		  XtNumber(about_buttons),
+		  XtNumber(about_buttons)-1);
   if (!XtIsManaged(about))
     popupByCursor(about, XtGrabNone);
 }
@@ -850,7 +865,7 @@ void aboutPopup(void)
 void setWMProps(Widget shell)
 {
 if (!XtIsShell(shell) || !XtIsRealized(shell)) {
-	String msg=XtMalloc(100);
+	String msg=XtMalloc(100+strlen(XtName(shell)));
 	sprintf(msg,"setWMProps(%s): need a realized shell",XtName(shell));
 	abortXfm(msg);
 	XtFree(msg); /* not really needed */
