@@ -1,8 +1,6 @@
 #ifndef _FocusFormP_h
 #define _FocusFormP_h
 /*
- *  $id$
- *
  *  The FocusForm Widget
  *  --------------------
  
@@ -20,16 +18,7 @@
 
  *  You should have received a copy of the GNU General Library Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- *  $Log: FocusFormP.h,v $
- *  Revision 1.1.1.1.4.1  2000/02/23 16:21:52  till
- *   - merged albert graef's changes from p3 to p4
- *
- *  Revision 1.2  1998/04/19 11:00:25  till
- *  started cvs logging
- *
-
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /* This is a subclass of the athena 'Form' widget */
@@ -47,7 +36,7 @@ all copies or substantial portions of the Software.
 */
 
 #include "FocusForm.h"
-#include <X11/Xaw/FormP.h>
+#include <X11/Xaw3d/FormP.h>
 
 typedef struct _FocusFormClassRec {
     CoreClassPart	core_class;
