@@ -278,50 +278,33 @@ static XtActionsRec actions[] =
 
 /* default translations */
 
-static char translations[] ="\
-			<Enter>:		CursorState(Active)\n\
-			<Leave>:		CursorState(Inactive)\n\
-	None	<Btn1Down>:		MoveCursor()\
-							HighlightStart()\n\
-			<Btn1Motion>:	MoveCursor()\
-							HighlightExtend()\n\
-			<Btn1Up>:		HighlightExtend()\
-							MakeSelection(PRIMARY)\n\
-			<Btn2Up>:		MoveCursor()\
-							InsertSelection(PRIMARY)\n\
-	Shift	<Key>Right:		MoveCursor()\
-							HighlightExtend()\
-							MakeSelection(PRIMARY)\n\
- 	Shift	<Key>Home:		MoveCursor(Home)\
-							HighlightExtend()\
-							MakeSelection(PRIMARY)\n\
- 	Shift	<Key>End:		MoveCursor(End)\
-							HighlightExtend()\
-							MakeSelection(PRIMARY)\n\
- 	Shift	<Key>Left:		MoveCursor(-1)\
-							HighlightExtend()\
-							MakeSelection(PRIMARY)\n\
- 	Shift	<Key>Delete:	Delete(Selection,End)    \n\
- 	Shift	<Key>BackSpace:	Delete(Selection,Home)  \n\
- 	Alt		<Key>Delete:	Delete(Selection,All)    \n\
- 	Alt		<Key>BackSpace:	Delete(Selection,All)  \n\
-	Ctrl	<Key>U:			Delete(All)	\n\
-			<Key>Right:		MoveCursor()	\
-							HighlightStart()\n\
- 			<Key>Home:		MoveCursor(Home)\
-							HighlightStart()\n\
- 			<Key>End:		MoveCursor(End) \
-							HighlightStart()\n\
- 			<Key>Left:		MoveCursor(-1)  \
-							HighlightStart()\n\
- 			<Key>Delete:	Delete(Selection,1)    \n\
- 			<Key>BackSpace:	Delete(Selection,-1)  \n\
-			<Key>SunCopy:	MakeSelection(CLIPBOARD)\n\
-			<Key>SunPaste:	InsertSelection(CLIPBOARD)\n\
-			<Key>SunCut:	DeleteToClipboard()\n\
-			<Key>Escape:	Restore()\n\
-			<Key>:			InsertChar()\
-";
+static char translations[] =
+"<Enter>:		CursorState(Active)\n"
+"<Leave>:		CursorState(Inactive)\n"
+"None<Btn1Down>:	MoveCursor() HighlightStart()\n"
+"<Btn1Motion>:		MoveCursor() HighlightExtend()\n"
+"<Btn1Up>:		HighlightExtend() MakeSelection(PRIMARY)\n"
+"<Btn2Up>:		MoveCursor() InsertSelection(PRIMARY)\n"
+"Shift<Key>Right:	MoveCursor() HighlightExtend() MakeSelection(PRIMARY)\n"
+"Shift<Key>Home:	MoveCursor(Home) HighlightExtend() MakeSelection(PRIMARY)\n"
+"Shift<Key>End:		MoveCursor(End) HighlightExtend() MakeSelection(PRIMARY)\n"
+"Shift<Key>Left:	MoveCursor(-1) HighlightExtend() MakeSelection(PRIMARY)\n"
+"Shift<Key>Delete:	Delete(Selection,End)\n"
+"Shift<Key>BackSpace:	Delete(Selection,Home)\n"
+"Alt<Key>Delete:	Delete(Selection,All)\n"
+"Alt<Key>BackSpace:	Delete(Selection,All)\n"
+"Ctrl<Key>U:		Delete(All)\n"
+"<Key>Right:		MoveCursor() HighlightStart()\n"
+"<Key>Home:		MoveCursor(Home) HighlightStart()\n"
+"<Key>End:		MoveCursor(End) HighlightStart()\n"
+"<Key>Left:		MoveCursor(-1)  HighlightStart()\n"
+"<Key>Delete:		Delete(Selection,1)    \n"
+"<Key>BackSpace:	Delete(Selection,-1)  \n"
+"<Key>SunCopy:		MakeSelection(CLIPBOARD)\n"
+"<Key>SunPaste:		InsertSelection(CLIPBOARD)\n"
+"<Key>SunCut:		DeleteToClipboard()\n"
+"<Key>Escape:		Restore()\n"
+"<Key>:			InsertChar()";
 
 /* Methods */
 
