@@ -2,12 +2,11 @@
 #define _TextFieldP_h
 
 /*
- *  $Id: TextFieldP.h,v 1.1.1.1.4.1 2000/02/23 16:21:52 till Exp $
- *
  *  The TextField Widget
  *  --------------------
  
  *  Copyright (C) 1997  by Till Straumann   <strauman@sun6hft.ee.tu-berlin.de>
+ *  modified 2006 by Bernhard R. Link (see Changelog for details)
 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Library Public License as published by
@@ -21,52 +20,18 @@
 
  *  You should have received a copy of the GNU General Library Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- *  $Log: TextFieldP.h,v $
- *  Revision 1.1.1.1.4.1  2000/02/23 16:21:52  till
- *   - merged albert graef's changes from p3 to p4
- *
- *  Revision 1.8  1999/10/28 18:09:10  cvs
- *   - userData needs #ifndef, not #ifdef!!!
- *
- *  Revision 1.7  1999/10/22 23:32:42  cvs
- *   - added the showNChars resource.
- *
- *  Revision 1.6  1999/10/21 15:28:16  cvs
- *   - added support for a label to TextField.
- *
- *  Revision 1.5  1999/02/05 17:41:54  till
- *   - added modifiedCallback reason (callData parameter).
- *
- *  Revision 1.4  1999/02/04 23:02:11  cvs
- *  - 	Added blinker / modifiedByUser.
- *  	Added the 'autoBlink' 'Commit()' action feature.
- *
- *  Revision 1.3  1998/10/30 22:18:58  till
- *  -fixed some bugs in TextField
- *  -C++ compliance for TextField.h
- *  -replace selected part of string
- *
- *  Revision 1.2  1998/04/19 11:00:25  till
- *  started cvs logging
- *
-
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "TextField.h"
 
 #ifdef TEXT_COMPAT
-#include <X11/Xaw/TextP.h> /* need XtNRResizeMode */
+#include <X11/Xaw3d/TextP.h> /* need XtNRResizeMode */
 #endif
 
 /* include superclass private header file */
 
-#ifdef XAW3D
 #include <X11/Xaw3d/ThreeDP.h>
-#else
-#include <X11/Xaw/SimpleP.h>
-#endif
 
 /* define unique representation types not found in <X11/StringDefs.h> */
 
@@ -77,9 +42,7 @@ typedef struct {
 typedef struct _TextFieldClassRec {
     CoreClassPart	core_class;
     SimpleClassPart	simple_class;
-#ifdef XAW3D
     ThreeDClassPart	threeD_class;
-#endif
     TextFieldClassPart	textField_class;
 } TextFieldClassRec;
 
@@ -90,7 +53,7 @@ typedef XawTextResizeMode TfResizeMode;
 #else
 typedef union {
 	Boolean b;
-	enum { a,b,c,d } pad;
+	enum { pas_a,pad_b,pad_c,pad_d } pad;
 } TfResizeMode;
 #endif
 
@@ -106,7 +69,7 @@ typedef struct {
     String			string;
     Boolean			use_string_in_place;
     XtJustify		justify;
-    int				maxlen;
+    Cardinal			maxlen;
     Boolean			readonly;
     short			scroll_chars;
     XtCallbackList	notify_cbl,modified_cbl;
@@ -130,10 +93,6 @@ typedef struct {
 	Dimension		labelWidth;
 	XtJustify		lblJustify;
 	int				showNChars;
-#ifndef XAW3D
-	XtPointer		userData;
-#endif
-
 
     /* private state */
 
@@ -153,9 +112,7 @@ typedef struct {
 typedef struct _TextFieldRec {
     CorePart		core;
     SimplePart		simple;
-#ifdef XAW3D
     ThreeDPart		threeD;
-#endif
     TextFieldPart	textField;
 } TextFieldRec;
 
