@@ -110,6 +110,8 @@ static char translations[] =
 "<Btn2Down>,<Btn2Up>	: fileToggle()\n"
 "<Btn2Down>,<Motion>	: fileBeginDrag(2,copy)\n"
 "Any<Btn3Down>		: grabMouseForPopup(dispatchDirFile,dirPopup,filePopup,\"\")\n"
+"Any<Btn4Down>		: ViewportScroll(-0.1,V)\n"
+"Any<Btn5Down>		: ViewportScroll(+0.1,V)\n"
 "Shift   <Key>n		: notify(*folder*new)\n"
 "Shift   <Key>g		: notify(*folder*goto)\n"
 "Shift   <Key>h		: notify(*folder*home)\n"
