@@ -1,9 +1,10 @@
 /*
+ *  This file is part of X-File Manager XFM
+ *  ----------------------------------------------------------------------
  *  Enhancements to the X-File Manager XFM-1.3.2 (Xt Helper functions)
- *  ------------------------------------------------------------------
  
  *  Copyright (C) 1997  by Till Straumann   <strauman@sun6hft.ee.tu-berlin.de>
-
+ *  ----------------------------------------------------------------------
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -16,10 +17,11 @@
 
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include <xfmconfig.h>
+
 /* Xt Helper Functions */
 
 #include "XtHelper.h"
