@@ -393,9 +393,9 @@ void magic_parse_singlefile(char *name, typeinitfunc *initfunc, size_t maxlen)
 }
 
 #ifdef HAVE_ZLIB
-// TODO: add resource to not look into those 
+// TODO: add resource to not look into those
 // (for too slow computers)
-static char *ungzip(int fd, unsigned char *inbuf, size_t maxhdr, size_t *bytes) {
+static char *ungzip(int fd, unsigned char *inbuf, size_t maxhdr, int *bytes) {
 	z_stream z;
 	unsigned char *outbuf;
 	unsigned char flags = hdrbuf[3];
@@ -492,11 +492,11 @@ static char *ungzip(int fd, unsigned char *inbuf, size_t maxhdr, size_t *bytes)
 		outbuf = NULL;
 	}
 	inflateEnd(&z);
-	return outbuf;
+	return (char*)outbuf;
 }
 #endif
 #ifdef HAVE_LIBBZ2
-static char *unbzip2(int fd, unsigned char *inbuf, size_t maxhdr, size_t *bytes) {
+static char *unbzip2(int fd, unsigned char *inbuf, size_t maxhdr, int *bytes) {
 	bz_stream bz;
 	char *outbuf;
 	char *extrabuf = NULL;
@@ -616,7 +616,7 @@ void magic_get_type(const char *name, const char **messagep, struct mime_filetyp
 		bytes = read(fd, hdrbuf, maxhdr);
 #ifdef HAVE_ZLIB
     		/* special treatment to look inside .gz files */
-		newbuffer = ungzip(fd,hdrbuf,maxhdr,&bytes);
+		newbuffer = ungzip(fd,(unsigned char*)hdrbuf,maxhdr,&bytes);
 		if( newbuffer != NULL ) {
 			size_t l;
 
