@@ -24,7 +24,6 @@
 
 #include <xfmconfig.h>
 
-#define _GNU_SOURCE 1 // for strndup
 #include <errno.h>
 #include <assert.h>
 #include <stdio.h>
@@ -610,6 +609,12 @@ static int mime_parse_line(const char *filename,int *linenr,FILE *f) {
 		if( e == '=' ) {
 			char *v;
 			e = mime_read_data(linenr,&colnr,&v,f);
+			if( e < 0 ) {
+				fputs("Out of memory!\n", stderr);
+				mime_read_skipline(linenr,&colnr,f);
+				mime_free(type);
+				return 0;
+			}
 
 			if( strcmp(var,"notes") == 0 ) {
 				if( mime_savememory )
